package org.apache.bcel.util;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import org.apache.commons.lang3.StringUtils;

public class JavaWrapper
{
    private final ClassLoader loader;
    
    private static ClassLoader getClassLoader() {
        final String s = System.getProperty("bcel.classloader");
        if (StringUtils.isEmpty((CharSequence)s)) {
            throw new IllegalStateException("The property 'bcel.classloader' must be defined");
        }
        try {
            return (ClassLoader)Class.forName(s).newInstance();
        }
        catch (final Exception e) {
            throw new IllegalStateException(e.toString(), e);
        }
    }
    
    public static void main(final String[] argv) throws Exception {
        if (argv.length == 0) {
            System.out.println("Missing class name.");
            return;
        }
        final String className = argv[0];
        final String[] newArgv = new String[argv.length - 1];
        System.arraycopy(argv, 1, newArgv, 0, newArgv.length);
        new JavaWrapper().runMain(className, newArgv);
    }
    
    public JavaWrapper() {
        this(getClassLoader());
    }
    
    public JavaWrapper(final ClassLoader loader) {
        this.loader = loader;
    }
    
    public void runMain(final String className, final String[] argv) throws ClassNotFoundException {
        final Class<?> cl = this.loader.loadClass(className);
        Method method = null;
        try {
            method = cl.getMethod("main", argv.getClass());
            final int m = method.getModifiers();
            final Class<?> r = method.getReturnType();
            if (!Modifier.isPublic(m) || !Modifier.isStatic(m) || Modifier.isAbstract(m) || r != Void.TYPE) {
                throw new NoSuchMethodException();
            }
        }
        catch (final NoSuchMethodException no) {
            System.out.println("In class " + className + ": public static void main(String[] argv) is not defined");
            return;
        }
        try {
            method.invoke(null, (Object[])argv);
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
    }
}
